SystemOrganization addCategory: #'PVpub-AnalogClockPrototype04'!

CircleMorph subclass: #Dial04
	instanceVariableNames: 'boundsLag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PVpub-AnalogClockPrototype04'!

!Dial04 commentStamp: 'pv 1/7/2014 01:03' prior: 0!
* Introduction to AnalogClock
This is the project manual for the AnalogClock tutorial.

It has been written in Org. This is an emacs-mode which helps organizing projects.

You can fetch the sources from [http://ss3.gemstone.com/ss/AnalogClock SqueakSource-AnalogClock]
Before loading PVpub-AnalogClockPrototype05 into your image you should create two classes:
in Squeak:
- Graphics-Primitives/LayoutConcentricFrame
- Morphic-Layouts/ConcentricLayout.
in Pharo:
- Morphic-Layouts/LayoutConcentricFrame
- Morphic-Layouts/ConcentricLayout

The sources come with the implementation of these two classes and 
small extensions for: 
- MorphicKernel/Morph>>layoutConcentricallyIn:, 
- Morphic-Layouts/LayoutPolicy>>isConcentricLayout.

If you want to start AnalogClock, open a workspace and execute
#+NAME: Compute Instance-LOC and Class-LOC
#+BEGIN_SRC Smalltalk
Dial05 new openInWorld.
#+END_SRC 

This project manual comes with the first six chapters built along a symmetry axis.
The first three chapters treat the translation steps from Product Requirements
to Architecture and Design and to Implementation. The next three chapters
comment the verification actions taken for the intermediate products of 
chapters one to three.
* Product Requirements
** Functional
*** Show an analog clock in Squeak
Show an analog clock with hour and minute watchhand.
:PROPERTIES:
:CUSTOM_ID: REQ101
:END:
*** Show a rotating watchhand
Show a PolygonMorph which rotates.
:PROPERTIES:
:CUSTOM_ID: REQ106
:END:
*** Show a rotating watchhand on a dial
Show a PolygonMorph which rotates on a dial formed like a circle.
:PROPERTIES:
:CUSTOM_ID: REQ107
:END:
*** Show a static watchhand on a dial able to get resized
Show a PolygonMorph on a dial which adapts when the dial gets resized.
:PROPERTIES:
:CUSTOM_ID: REQ108
:END:
*** Show a rotating watchhand on a dial able to get resized
Show a PolygonMorph rotating on a dial which adapts when the dial gets resized.
:PROPERTIES:
:CUSTOM_ID: REQ109
:END:
*** Synchronize clock to the OS time
The time shown on the AnalogClock is the same as the time of the OS we are running on.
:PROPERTIES:
:CUSTOM_ID: REQ102
:END:
*** Respond to the Pick up-Halo
The AnalogClock may be moved around. All its submorphs are included in an adequate manner.
:PROPERTIES:
:CUSTOM_ID: REQ103
:END:
*** Respond to the Resize-Halo
The AnalogClock may get resized. All its submorphs are included in an adequate manner.
:PROPERTIES:
:CUSTOM_ID: REQ104
:END:
*** Be able to embed one AnalogClock in another
The AnalogClock may get embedded into another one.
:PROPERTIES:
:CUSTOM_ID: REQ105
:END:
*** Do not show elements which are too small to recognize them
When resizing makes the clock get too small, do not show the watchhand anymore.
:PROPERTIES:
:CUSTOM_ID: REQ112
:END:
*** Show three watchhands: secons, minutes and hours
When resizing makes the clock get too small, do not show the watchhand anymore.
:PROPERTIES:
:CUSTOM_ID: REQ113
:END:
** Non-Functional
*** Store project information in Squeak
Show information about the application lifecycle in the class comment. Use org-mode-format.
:PROPERTIES:
:CUSTOM_ID: REQ110
:END:
*** Be able to run in Squeak and in Pharo
Be platform independent and run in Squeak and in Pharo.
:PROPERTIES:
:CUSTOM_ID: REQ114
:END:
** Process
*** Write unit tests
:PROPERTIES:
:CUSTOM_ID: REQ111
:END:
** Tools
* Architecture and Design
** abc
*** Design Decision: Avoid doing everything in drawOn:
[http://www.squeaksource.com/WatchMorph WatchMorph] relies on drawOn:. With each time step and on each pich up- or resize-operation the whole watch is drawn.
This seems ok for a simple watch. But it is not scalable.
Decision <2013-11-24 Sun>: adapt Morphic's Layout-mechanism (TableLayout).
Traces to: [[#REQ108]]
:PROPERTIES:
:CUSTOM_ID: DES001
:END:
*** Design Decision: Hide implementation details of two watchhands for resize-operation
We use two watchhands to implement correct resize behaviour. The interface for a Dial should get concentrated in one class.
Decision <2013-12-02 Mon>: create ResizableWatchhand.
Traces to: [[#REQ109]]
:PROPERTIES:
:CUSTOM_ID: DES002
:END:
*** Design Decision: Create a control interface for the clock
Create a class which acts like an interface for a user control panel and for tests.
Decision <2013-12-04 Tue>: create Dial05Interface.
Traces to: 
:PROPERTIES:
:CUSTOM_ID: DES003
:END:
*** Design Decision: Hide implementation details of two watchhands for resize-operation by using 2 objects instead of 3
We use two watchhands to implement correct resize behaviour. The interface for a Dial should get concentrated in one class.
Use one additional object to implement the midnight watchhand.
Decision <2013-12-27 Tue>: create Watchhand06 (visible rotating watchhand) and its member variable watchhand06Midnight (invisible static watchhand).
Traces to: 
:PROPERTIES:
:CUSTOM_ID: DES004
:END:
* Implementation
** Step 1
*** PVpub-AnalogClockPrototype01 (WatchHand01)
Shows a rotating PolygonMorph.
:PROPERTIES:
:CUSTOM_ID: IMP001
:END:
** Step 2
*** PVpub-AnalogClockPrototype02 (Dial02, WatchHand02)
Shows a rotating watchhand (PolygonMorph) rotating on a dial (CircleMorph). When resizing the clock, the watchhand does not adapt.
:PROPERTIES:
:CUSTOM_ID: IMP002
:END:
** Step 3
*** PVpub-AnalogClockPrototype03 (Dial03, WatchHand03)
Shows a non-rotating watchhand (PolygonMorph) on a dial (CircleMorph). When resizing the clock, the watchhand does adapt.
:PROPERTIES:
:CUSTOM_ID: IMP003
:END:
** Step 4
*** PVpub-AnalogClockPrototype04 (Dial04, WatchHand04)
Shows a rotating watchhand (PolygonMorph) on a dial (CircleMorph). When resizing the clock, the watchhand does adapt.
:PROPERTIES:
:CUSTOM_ID: IMP004
:END:
** Step 5
*** PVpub-AnalogClockPrototype05 (Dial05, ResizableWatchHand05)
Shows a rotating watchhand (PolygonMorph) on a dial (CircleMorph). When resizing the clock, the watchhand does adapt. The interplay between Dial and WatchHand is simplified. An analog clock may get embedded into another.
:PROPERTIES:
:CUSTOM_ID: IMP005
:END:
** Step 6
*** PVpub-AnalogClockPrototype06 (Dial06, Watchhand06)
Shows a rotating watchhand (Watchhand06) on a dial (CircleMorph). When resizing the clock, the watchhand does adapt. Compared to Step 5 the design is simplified.
:PROPERTIES:
:CUSTOM_ID: IMP006
:END:
* Verification of Implementation
* Verification at Architecture Level
* Verification at Product Level
** Versions and Releases
*** PVpub-AnalogClockPrototype01
**** VER001
abc
***** Implemented Requirements
[[#REQ106]]
***** Metrics
:PROPERTIES:
:SWA-LINT_ClassNotReferenced: 3
:INSTANCE-LOC:                11
:CLASS-LOC:                   0
:MCZ:                         PVpub-AnalogClockPrototype01-pv.1
:END:
*** PVpub-AnalogClockPrototype02
**** VER001
abc
***** Implemented Requirements
[[#REQ107]]
***** Metrics
:PROPERTIES:
:SWA-LINT_ClassNotReferenced: 1
:SWA-Lint_InconsistentMethodClassification: 1
:INSTANCE-LOC:                35
:CLASS-LOC:                   0
:MCZ:                         PVpub-AnalogClockPrototype02-pv.1
:END:
*** PVpub-AnalogClockPrototype03
**** VER001
abc
***** Implemented Requirements
[[#REQ108]]
***** Metrics
:PROPERTIES:
:SWA-LINT_ClassNotReferenced: 1
:SWA-Lint_InconsistentMethodClassification: 1
:INSTANCE-LOC:                32
:CLASS-LOC:                   0
:MCZ:                         PVpub-AnalogClockPrototype03-pv.1
:END:
*** PVpub-AnalogClockPrototype04
**** VER001
Dial has two members. watchhand04m is a watchhand in midnight position. watchhand04
is a rotating watchhand.
***** Implemented Requirements
[[#REQ109]]
***** Metrics
:PROPERTIES:
:CUSTOM_ID:                   VER-004-001
:SWA-LINT_ClassNotReferenced: 2
:SWA-Lint_InconsistentMethodClassification: 2
:INSTANCE-LOC:                84
:CLASS-LOC:                   6
:MCZ:                         PVpub-AnalogClockPrototype04-pv.1
:END:
**** VER002
Some simplifications.
***** Implemented Requirements
Same as VER001
***** Metrics
:PROPERTIES:
:CUSTOM_ID:                   VER-004-002
:SWA-LINT_ClassNotReferenced: 2
:SWA-Lint_InconsistentMethodClassification: 2
:INSTANCE-LOC:                84
:CLASS-LOC:                   6
:MCZ:                         PVpub-AnalogClockPrototype04-pv.2
:END:
*** PVpub-AnalogClockPrototype05
**** VER001
abc
***** Implemented Requirements
[[#REQ109]]
***** Implemented Design Decision
[[#DES002]]
***** Metrics
:PROPERTIES:
:SWA-LINT_ClassNotReferenced: 3
:SWA-Lint_InconsistentMethodClassification: 2
:INSTANCE-LOC:                108
:CLASS-LOC:                   6
:MCZ:                         PVpub-AnalogClockPrototype05-pv.1
:END:
**** VER002
Add class comment (org-mode)
***** Implemented Requirements
[[#REQ110]]
***** Metrics
:PROPERTIES:
:SWA-LINT_ClassNotReferenced: 3
:SWA-Lint_InconsistentMethodClassification: 2
:INSTANCE-LOC:                108
:CLASS-LOC:                   6
:MCZ:                         PVpub-AnalogClockPrototype05-pv.2
:MCZ-TIME:                    Time: 2 December 2013, 10:33:09.286 pm
:END:
**** VER003
renaming
***** Implemented Requirements
none
***** Metrics
:PROPERTIES:
:SWA-LINT_ClassNotReferenced: 3
:SWA-Lint_InconsistentMethodClassification: 2
:INSTANCE-LOC:                112
:CLASS-LOC:                   11
:MCZ:                         PVpub-AnalogClockPrototype05-pv.3
:MCZ-TIME:                    Time: 4 December 2013, 10:38:15.566 pm
:END:
**** VER004
Control interface
***** Implemented Requirements
implement [[#DES003]]
***** Metrics
:PROPERTIES:
:SWA-LINT_MethodsImplementedButNotSent: 2
:SWA-LINT_VariableReferencedInOnlyOneMethod: 2
:SWA-LINT_ClassNotReferenced: 5
:SWA-Lint_InconsistentMethodClassification: 2
:INSTANCE-LOC:                122
:CLASS-LOC:                   11
:MCZ:                         PVpub-AnalogClockPrototype05-pv.4
:MCZ-TIME:                    Time: 4 December 2013, 11:15:51.485 pm
:END:
**** VER005
Add unit tests
***** Implemented Requirements
implement [[#REQ111]]
***** Metrics PVpub-AnalogClockPrototype05
:PROPERTIES:
:SWA-LINT_MethodsImplementedButNotSent: 1
:SWA-Lint_InconsistentMethodClassification: 2
:INSTANCE-LOC:                139
:CLASS-LOC:                   11
:MCZ:                         PVpub-AnalogClockPrototype05-pv.5
:MCZ-TIME:                    Time: 5 December 2013, 9:41:27.372 pm
:END:
***** Metrics PVpubTests-AnalogClockPrototype05
:PROPERTIES:
:SWA-LINT_MethodsImplementedButNotSent: SwaLint terminates with a bug
:INSTANCE-LOC:                31
:CLASS-LOC:                   0
:MCZ:                         PVpubTests-AnalogClockPrototype05-pv.1
:MCZ-TIME:                    Time: 5 December 2013, 9:45:38.638 pm
:END:
**** VER006
Add current class comment
***** Implemented Requirements
implement [[#REQ111]]
***** Metrics PVpub-AnalogClockPrototype05
:PROPERTIES:
:SWA-LINT_MethodsImplementedButNotSent: 1
:SWA-Lint_InconsistentMethodClassification: 2
:INSTANCE-LOC:                139
:CLASS-LOC:                   11
:MCZ:                         PVpub-AnalogClockPrototype05-pv.6
:MCZ-TIME:                    Time: 5 December 2013, 9:59:41.815 pm
:END:
***** Metrics PVpubTests-AnalogClockPrototype05
same as for VER005
**** VER007
A first draft for [[REQ105]]
***** Implemented Requirements
implement [[#REQ105]]
***** Metrics PVpub-AnalogClockPrototype05
:PROPERTIES:
:MCZ:                         PVpub-AnalogClockPrototype05-pv.7
:MCZ-TIME:                    Time: 8 December 2013, 10:15:03.414 am
:END:
***** Metrics PVpubTests-AnalogClockPrototype05
same as for VER005
**** VER008
***** Implemented Requirements
Fix [[#BUG101]]
***** Metrics PVpub-AnalogClockPrototype05
:PROPERTIES:
:CUSTOM_ID:                   VER-005-008
:MCZ:                         PVpub-AnalogClockPrototype05-pv.8
:MCZ-TIME:                    Time: 26 December 2013, 1:54:15.664 am 
:END:
***** Metrics PVpubTests-AnalogClockPrototype05
same as for VER005
**** VER009
***** Implemented Requirements
Minor changes
***** Metrics PVpub-AnalogClockPrototype05
:PROPERTIES:
:CUSTOM_ID:                   VER-005-009
:MCZ:                         PVpub-AnalogClockPrototype05-pv.9
:MCZ-TIME:                    Time: 3 January 2014, 11:09:36.067 pm 
:END:
***** Metrics PVpubTests-AnalogClockPrototype05
same as for VER005
*** PVpub-AnalogClockPrototype06
**** VER001
Design modification with respect to PVpub-AnalogClockPrototype05
***** Implemented Requirements
Comparable to those of PVpub-AnalogClockPrototype04
***** Implemented Design Decision
[[#DES004]]
***** Metrics
:PROPERTIES:
:CUSTOM_ID:                   VER-006-001
:SWA-LINT_VariableReferencedInOnly: 2
:SWA-LINT_ClassNotReferenced: 2
:SWA-Lint_InconsistentMethodClassification: 2
:INSTANCE-LOC:                69
:CLASS-LOC:                   6
:MCZ:                         PVpub-AnalogClockPrototype06-pv.1
:MCZ-TIME:                    27 December 2013, 6:01:36.802 pm
:END:
**** VER002
Minor changes to improve readability
***** Implemented New Requirements
[[REQ113]]
***** Metrics
:PROPERTIES:
:CUSTOM_ID:                   VER-006-002
:SWA-LINT_DebuggingCodeLeftInMethods: 2
:SWA-LINT_ClassNotReferenced: 5
:SWA-Lint_InconsistentMethodClassification: 2
:INSTANCE-LOC:                94
:CLASS-LOC:                   2
:MCZ:                         PVpub-AnalogClockPrototype06-pv.2
:MCZ-TIME:                    2 January 2014, 3:47:09.88 pm 
:END:
** Bugs
*** 
**** BUG101 - Extension for CircularLayout is missing
:PROPERTIES:
:CUSTOM_ID: BUG101
:PREPARE: Someone who wants to install Dial05 will run into problems. He needs several extensions.
These extensions 
:BUG: are not 
:EXPECTED: should be
:END: part of the PVpub-AnalogClockPrototype05 package.
:FOUND: 2013-12-08
:SOLVED: 2013-12-26
:END:
**** TODO BUG102 - Resize is not doing well when embedding a clock into another
:PROPERTIES:
:CUSTOM_ID: BUG102
:PREPARE: Embed one clock into another with "embed into". Resize. 
The embedded clock
:BUG: is not 
:EXPECTED: is
:END: aligned with the embedding one.
:FOUND: 2013-12-26
:SOLVED: 2013-12-
:END:
**** TODO [#B] BUG103 - use CircularLayout to embed one clock in another
:PROPERTIES:
:CUSTOM_ID: BUG103
:PREPARE: Look at the design. The layout between a clock and another embedded into it is done via
:BUG: clumsy computations in drawOn: 
:EXPECTED: CircularLayout
:END: .
:FOUND: 2013-12-27
:SOLVED: 2013-12-
:END:
**** TODO [#C] BUG104 - Show UML diagrams as part of this text
:PROPERTIES:
:CUSTOM_ID: BUG104
:PREPARE: Read this text, especially the section "Design Decisions". It
:BUG: is not 
:EXPECTED: is
:END: possible to show simple UML diagrams here which can be exported to LaTeX & DocBook.
:FOUND: 2013-12-27
:SOLVED: 2013-12-
:END:
**** TODO [#A] BUG105 - Avoid redraw of watchhand with slightly different vertices which could have been left unchanged
:PROPERTIES:
:CUSTOM_ID: BUG105
:PREPARE: Let the clock do one second-step. Look at the minutes watchhand or the hours watchhand.
Try different clock sizes. You will find  
:BUG: a size where intended changes on the second watchhand
lead to unintended slight position changes 
:Expected: no size where intended changes on the second watchhand
lead to unintended slight position changes
:END: on the minutes or hours watchhand.
:FOUND: VER-006-002
:SOLVED: 
:END:
**** TODO [#A] BUG106 - Center of watchhand is not at crossing of the diagonals of lozenge.
:PROPERTIES:
:CUSTOM_ID: BUG106
:PREPARE: Resize clock to a small size. Display the rotation center.
You will find that the rotation center 
:BUG: is not at
:Expected: is at
:END: the crossing point of the diagonals of the lozenge.
:FOUND: VER-006-002
:SOLVED: 
:END:
* Appendix
** Code snippets
*** Compute Instance-LOC and Class-LOC
#+NAME: Compute Instance-LOC and Class-LOC
#+BEGIN_SRC Smalltalk
| instanceLoc classLoc |
instanceLoc := classLoc := 0.
(PackageInfo named: 'PVpubTests-AnalogClockPrototype05') classes
	do: [:class | 
		instanceLoc := class linesOfCode + instanceLoc.
		classLoc := class class linesOfCode + classLoc].
Transcript show: 'instanceLoc=' , instanceLoc asString , ', classLoc=' , classLoc asString.
#+END_SRC


!

!Dial04 methodsFor: 'drawing' stamp: 'pv 12/1/2013 21:37'!
drawOn: aCanvas 
	| currentExtent oldExtent |
	currentExtent := self bounds corner - self bounds origin.
	oldExtent := boundsLag corner - boundsLag origin.
	currentExtent ~= oldExtent
		ifTrue: [self submorphs
				do: [:each | each isMovingWatchhand
						ifTrue: [each step]].
			boundsLag := self bounds].
	super drawOn: aCanvas! !

!Dial04 methodsFor: 'initialization' stamp: 'pv 12/1/2013 21:38'!
initialize
	| watchHand04 watchHand04m |
	super initialize. 
	self extent: 200 @ 200;
		 position: 300 @ 300;
		 layoutPolicy: ConcentricLayout new.
	"watchhand in midnight position"
	watchHand04m := WatchHand04 new.
	watchHand04m
		setInitialMidnightVerticesFromCenter: self center
		andRadius: self extent x / 2
		withFraction: 0.9;
		 visible: false.
	self
		addMorph: watchHand04m
		fullFrame: (LayoutConcentricFrame
				left: 0.3
				right: 0.3
				top: 0.9
				bottom: 0.3).
	"rotating watchhand"
	watchHand04 := WatchHand04 new.
	watchHand04
		setInitialMidnightVerticesFromCenter: self center
		andRadius: self extent x / 2
		withFraction: 0.9;
		 setMidnightWatchhand: watchHand04m.
	self addMorph: watchHand04.
	boundsLag := 1 @ 1 corner: 2 @ 2! !

PolygonMorph subclass: #WatchHand04
	instanceVariableNames: 'midnightVertices midnightWatchhand'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PVpub-AnalogClockPrototype04'!

!WatchHand04 class methodsFor: 'geometry' stamp: 'pv 12/1/2013 21:57'!
inDegrees: aTime 
	"080413 should be a class method."
	| timeValue delta |
	delta := 0.0.
	timeValue := (aTime perform: #seconds) asFloat.
	^ 360 * timeValue / 60! !

!WatchHand04 methodsFor: 'accessing - properties' stamp: 'pv 11/11/2013 23:10'!
isMovingWatchhand
	self layoutFrame
		ifNil: [^ true].
	^ false! !

!WatchHand04 methodsFor: 'geometry' stamp: 'pv 11/9/2013 22:33'!
rotationDegreesFromMidnightPosition: degrees 
	| flex center |
	(center := self valueOfProperty: #referencePosition)
		ifNil: [self setProperty: #referencePosition toValue: (center := self bounds center)].
	flex := (MorphicTransform offset: center negated)
				withAngle: degrees  degreesToRadians.
	self
		setVertices: (vertices
				collect: [:v | (flex transform: v)
						- flex offset]).
	self forwardDirection: degrees! !

!WatchHand04 methodsFor: 'initialization' stamp: 'pv 11/9/2013 22:13'!
setInitialMidnightVerticesFromCenter: aPoint andRadius: aFloat withFraction: aFractionFloat 
	"2013-09-29pv: vertices sind immer Float"
	| lozengeLengthFromCenter lozengeWidthFromCenter |
	lozengeLengthFromCenter := 3 max: aFloat * aFractionFloat.
	lozengeWidthFromCenter := 1 max: aFloat * (aFractionFloat / 3).
	midnightVertices := Array
				with: aPoint + (0 @ lozengeWidthFromCenter)
				with: aPoint + (-1 * lozengeWidthFromCenter @ 0)
				with: aPoint + (0 @ -1 * lozengeLengthFromCenter)
				with: aPoint + (lozengeWidthFromCenter @ 0).
	self setVertices: midnightVertices! !

!WatchHand04 methodsFor: 'initialization' stamp: 'pv 11/26/2013 19:46'!
setMidnightVerticesFrom: anArray
	midnightVertices := anArray.! !

!WatchHand04 methodsFor: 'initialization' stamp: 'pv 12/1/2013 21:27'!
setMidnightWatchhand: aWatchHand
	midnightWatchhand := aWatchHand.! !

!WatchHand04 methodsFor: 'stepping and presenter' stamp: 'pv 1/6/2014 23:24'!
step
	self isMovingWatchhand
		ifFalse: [^ self].
	vertices := midnightWatchhand vertices.
	self setRotationCenterFrom: owner center.
	self
		rotationDegreesFromMidnightPosition: (self class inDegrees: DateAndTime now)! !

!WatchHand04 methodsFor: 'stepping and presenter' stamp: 'pv 11/11/2013 22:32'!
stepTime
	^ 1000! !
